#ifndef lint
static char *RCSid = "$Header$";
#endif

/*
 * ifstatus - check for interfaces in promiscuous or debug mode
 *
 * David A. Curry
 * Purdue University
 * Engineering Computer Network
 * 1285 Electrical Engineering Building
 * West Lafayette, IN 47907-1285
 * davy@ecn.purdue.edu
 *
 * $Log$
 */

#include <sys/param.h>
#include <sys/socket.h>
#if defined(SUNOS4) || defined(SUNOS5)
#include <sys/sockio.h>
#endif
#include <sys/ioctl.h>
#include <net/if.h>
#include <ctype.h>
#include <stdio.h>

#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN		64
#endif

char	*pname = NULL;
int	verbose = 0;

main(argc, argv)
char **argv;
int argc;
{
	char *p;
	int n, s;
	char cbuf[BUFSIZ];
	struct ifconf ifc;
	struct ifreq ifr, *ifrp;
	char hostname[MAXHOSTNAMELEN];

	pname = *argv;

	/*
	 * Process arguments.
	 */
	while (--argc) {
		if (**++argv != '-')
			usage();

		switch (*++*argv) {
		case 'v':
			verbose++;
			break;
		default:
			usage();
			break;
		}
	}

	/*
	 * Get the local hostname.
	 */
	if (gethostname(hostname, sizeof(hostname)) < 0)
		fatal("gethostname", 0);

	/*
	 * Convert hostname to upper case (makes message prettier).
	 */
	for (p = hostname; *p != '\0'; p++) {
		if (islower(*p))
			*p = toupper(*p);
	}

	/*
	 * Get a socket.
	 */
	if ((s = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
		fatal("socket", 0);

	ifc.ifc_buf = cbuf;
	ifc.ifc_len = sizeof(cbuf);

	/*
	 * Get the list of configured interfaces.
	 */
	if (ioctl(s, SIOCGIFCONF, (char *) &ifc) < 0)
		fatal("ioctl: SIOCGIFCONF", 0);

	close(s);
	ifrp = ifc.ifc_req;

	/*
	 * For each interface...
	 */
	for (n = ifc.ifc_len / sizeof(struct ifreq); n > 0; n--, ifrp++) {
		/*
		 * We need a new socket for each interface.
		 */
		if ((s = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
			fatal("socket");

		strcpy(ifr.ifr_name, ifrp->ifr_name);

		if (verbose)
			printf("checking interface %s...", ifr.ifr_name);

		/*
		 * Get the flags on the interface.
		 */
		if (ioctl(s, SIOCGIFFLAGS, (char *) &ifr) < 0)
			fatal("ioctl: SIOCGIFFLAGS", ifr.ifr_name);

		if (verbose)
			printf(" flags = 0x%x\n", ifr.ifr_flags);

		/*
		 * Check for promiscuous mode.
		 */
		if (ifr.ifr_flags & IFF_PROMISC) {
			printf("WARNING: %s INTERFACE %s IS IN PROMISCUOUS MODE.\n",
			       hostname, ifr.ifr_name);
		}

		/*
		 * Check for debug mode.
		 */
		if (ifr.ifr_flags & IFF_DEBUG) {
			printf("WARNING: %s INTERFACE %s IS IN DEBUG MODE.\n",
			       hostname, ifr.ifr_name);
		}

		close(s);
	}

	exit(0);
}

/*
 * fatal - print an error message and exit.
 */
fatal(s1, s2)
char *s1, *s2;
{
	fprintf(stderr, "%s: ", pname);

	if (s2)
		fprintf(stderr, "%s: ", s2);

	perror(s1);
	exit(1);
}

/*
 * usage - print a usage message and exit.
 */
usage()
{
	fprintf(stderr, "Usage: %s [-v]\n", pname);
	exit(1);
}
